@@ -18,7 +18,6 @@ const Atk = imports.gi.Atk;
 const Config = imports.misc.config;
 const CtrlAltTab = imports.ui.ctrlAltTab;
 const DND = imports.ui.dnd;
-const Layout = imports.ui.layout;
 const Overview = imports.ui.overview;
 const PopupMenu = imports.ui.popupMenu;
 const PanelMenu = imports.ui.panelMenu;
@@ -75,33 +74,41 @@ function _unpremultiply(color) {
                                blue: blue, alpha: color.alpha });
 };
 
+const Animation = new Lang.Class({
+    Name: 'Animation',
 
-const AnimatedIcon = new Lang.Class({
-    Name: 'AnimatedIcon',
-
-    _init: function(name, size) {
-        this.actor = new St.Bin({ visible: false });
+    _init: function(filename, width, height, speed) {
+        this.actor = new St.Bin();
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
-        this.actor.connect('notify::visible', Lang.bind(this, this._onVisibleNotify));
+        this._speed = speed;
 
+        this._isLoaded = false;
+        this._isPlaying = false;
         this._timeoutId = 0;
         this._frame = 0;
-        this._animations = St.TextureCache.get_default().load_sliced_image (global.datadir + '/theme/' + name, size, size);
+        this._animations = St.TextureCache.get_default().load_sliced_image (filename, width, height,
+                                                                            Lang.bind(this, this._animationsLoaded));
         this.actor.set_child(this._animations);
     },
 
-    _disconnectTimeout: function() {
+    play: function() {
+        if (this._isLoaded && this._timeoutId == 0) {
+            if (this._frame == 0)
+                this._showFrame(0);
+
+            this._timeoutId = Mainloop.timeout_add(this._speed, Lang.bind(this, this._update));
+        }
+
+        this._isPlaying = true;
+    },
+
+    stop: function() {
         if (this._timeoutId > 0) {
             Mainloop.source_remove(this._timeoutId);
             this._timeoutId = 0;
         }
-    },
 
-    _onVisibleNotify: function() {
-        if (this.actor.visible)
-            this._timeoutId = Mainloop.timeout_add(ANIMATED_ICON_UPDATE_TIMEOUT, Lang.bind(this, this._update));
-        else
-            this._disconnectTimeout();
+        this._isPlaying = false;
     },
 
     _showFrame: function(frame) {
@@ -121,8 +128,24 @@ const AnimatedIcon = new Lang.Class({
         return true;
     },
 
+    _animationsLoaded: function() {
+        this._isLoaded = true;
+
+        if (this._isPlaying)
+            this.play();
+    },
+
     _onDestroy: function() {
-        this._disconnectTimeout();
+        this.stop();
+    }
+});
+
+const AnimatedIcon = new Lang.Class({
+    Name: 'AnimatedIcon',
+    Extends: Animation,
+
+    _init: function(filename, size) {
+        this.parent(filename, size, size, ANIMATED_ICON_UPDATE_TIMEOUT);
     }
 });
 
@@ -179,7 +202,7 @@ const TextShadower = new Lang.Class({
             let child = children[i];
             let childBox = new Clutter.ActorBox();
             // The order of the labels here is arbitrary, except
-            // we know the "real" label is at the end because Clutter.Group
+            // we know the "real" label is at the end because Clutter.Actor
             // sorts by Z order
             switch (i) {
                 case 0: // top
@@ -235,6 +258,7 @@ const AppMenuButton = new Lang.Class({
         this._actionGroupNotifyId = 0;
 
         let bin = new St.Bin({ name: 'appMenu' });
+        bin.connect('style-changed', Lang.bind(this, this._onStyleChanged));
         this.actor.add_actor(bin);
 
         this.actor.bind_property("reactive", this.actor, "can-focus", 0);
@@ -247,6 +271,10 @@ const AppMenuButton = new Lang.Class({
         this._container.connect('get-preferred-height', Lang.bind(this, this._getContentPreferredHeight));
         this._container.connect('allocate', Lang.bind(this, this._contentAllocate));
 
+        let textureCache = St.TextureCache.get_default();
+        textureCache.connect('icon-theme-changed',
+                             Lang.bind(this, this._onIconThemeChanged));
+
         this._iconBox = new Shell.Slicer({ name: 'appMenuIcon' });
         this._iconBox.connect('style-changed',
                               Lang.bind(this, this._onIconBoxStyleChanged));
@@ -261,32 +289,31 @@ const AppMenuButton = new Lang.Class({
         this._visible = !Main.overview.visible;
         if (!this._visible)
             this.actor.hide();
-        Main.overview.connect('hiding', Lang.bind(this, function () {
+        this._overviewHidingId = Main.overview.connect('hiding', Lang.bind(this, function () {
             this.show();
         }));
-        Main.overview.connect('showing', Lang.bind(this, function () {
+        this._overviewShowingId = Main.overview.connect('showing', Lang.bind(this, function () {
             this.hide();
         }));
 
         this._stop = true;
 
-        this._spinner = new AnimatedIcon('process-working.svg',
-                                         PANEL_ICON_SIZE);
-        this._container.add_actor(this._spinner.actor);
-        this._spinner.actor.lower_bottom();
+        this._spinner = null;
 
         let tracker = Shell.WindowTracker.get_default();
         let appSys = Shell.AppSystem.get_default();
-        tracker.connect('notify::focus-app', Lang.bind(this, this._focusAppChanged));
-        appSys.connect('app-state-changed', Lang.bind(this, this._onAppStateChanged));
-
-        global.window_manager.connect('switch-workspace', Lang.bind(this, this._sync));
+        this._focusAppNotifyId =
+            tracker.connect('notify::focus-app', Lang.bind(this, this._focusAppChanged));
+        this._appStateChangedSignalId =
+            appSys.connect('app-state-changed', Lang.bind(this, this._onAppStateChanged));
+        this._switchWorkspaceNotifyId =
+            global.window_manager.connect('switch-workspace', Lang.bind(this, this._sync));
 
         this._sync();
     },
 
     show: function() {
-        if (this._visible || Main.screenShield.locked)
+        if (this._visible)
             return;
 
         this._visible = true;
@@ -326,12 +353,33 @@ const AppMenuButton = new Lang.Class({
                            onCompleteScope: this });
     },
 
+    _onStyleChanged: function(actor) {
+        let node = actor.get_theme_node();
+        let [success, icon] = node.lookup_url('spinner-image', false);
+        if (!success || this._spinnerIcon == icon)
+            return;
+        this._spinnerIcon = icon;
+        this._spinner = new AnimatedIcon(this._spinnerIcon, PANEL_ICON_SIZE);
+        this._container.add_actor(this._spinner.actor);
+        this._spinner.actor.hide();
+        this._spinner.actor.lower_bottom();
+    },
+
     _onIconBoxStyleChanged: function() {
         let node = this._iconBox.get_theme_node();
         this._iconBottomClip = node.get_length('app-icon-bottom-clip');
         this._updateIconBoxClip();
     },
 
+    _onIconThemeChanged: function() {
+        if (this._iconBox.child == null)
+            return;
+
+        this._iconBox.child.destroy();
+        let icon = this._targetApp.get_faded_icon(2 * PANEL_ICON_SIZE);
+        this._iconBox.set_child(icon);
+    },
+
     _updateIconBoxClip: function() {
         let allocation = this._iconBox.allocation;
         if (this._iconBottomClip > 0)
@@ -348,12 +396,17 @@ const AppMenuButton = new Lang.Class({
 
         this._stop = true;
         this.actor.reactive = true;
+
+        if (this._spinner == null)
+            return;
+
         Tweener.addTween(this._spinner.actor,
                          { opacity: 0,
                            time: SPINNER_ANIMATION_TIME,
                            transition: "easeOutQuad",
                            onCompleteScope: this,
                            onComplete: function() {
+                               this._spinner.stop();
                                this._spinner.actor.opacity = 255;
                                this._spinner.actor.hide();
                            }
@@ -363,6 +416,11 @@ const AppMenuButton = new Lang.Class({
     startAnimation: function() {
         this._stop = false;
         this.actor.reactive = false;
+
+        if (this._spinner == null)
+            return;
+
+        this._spinner.play();
         this._spinner.actor.show();
     },
 
@@ -424,6 +482,9 @@ const AppMenuButton = new Lang.Class({
         }
         this._label.actor.allocate(childBox, flags);
 
+        if (this._spinner == null)
+            return;
+
         if (direction == Clutter.TextDirection.LTR) {
             childBox.x1 = Math.floor(iconWidth / 2) + this._label.actor.width;
             childBox.x2 = childBox.x1 + this._spinner.actor.width;
@@ -514,7 +575,8 @@ const AppMenuButton = new Lang.Class({
             return;
         }
 
-        this._spinner.actor.hide();
+        if (this._spinner)
+            this._spinner.actor.hide();
         if (this._iconBox.child != null)
             this._iconBox.child.destroy();
         this._iconBox.hide();
@@ -558,12 +620,18 @@ const AppMenuButton = new Lang.Class({
                 return;
 
             menu = new PopupMenu.RemoteMenu(this.actor, this._targetApp.menu, this._targetApp.action_group);
+            menu.connect('activate', Lang.bind(this, function() {
+                let win = this._targetApp.get_windows()[0];
+                win.check_alive(global.get_current_time());
+            }));
+
         } else {
-            if (this.menu && !(this.menu instanceof PopupMenu.RemoteMenu))
+            if (this.menu.isDummyQuitMenu)
                 return;
 
             // fallback to older menu
             menu = new PopupMenu.PopupMenu(this.actor, 0.0, St.Side.TOP, 0);
+            menu.isDummyQuitMenu = true;
             menu.addAction(_("Quit"), Lang.bind(this, function() {
                 this._targetApp.request_quit();
             }));
@@ -571,93 +639,71 @@ const AppMenuButton = new Lang.Class({
 
         this.setMenu(menu);
         this._menuManager.addMenu(menu);
+    },
+
+    destroy: function() {
+        if (this._appStateChangedSignalId > 0) {
+            let appSys = Shell.AppSystem.get_default();
+            appSys.disconnect(this._appStateChangedSignalId);
+            this._appStateChangedSignalId = 0;
+        }
+        if (this._focusAppNotifyId > 0) {
+            let tracker = Shell.WindowTracker.get_default();
+            tracker.disconnect(this._focusAppNotifyId);
+            this._focusAppNotifyId = 0;
+        }
+        if (this._overviewHidingId > 0) {
+            Main.overview.disconnect(this._overviewHidingId);
+            this._overviewHidingId = 0;
+        }
+        if (this._overviewShowingId > 0) {
+            Main.overview.disconnect(this._overviewShowingId);
+            this._overviewShowingId = 0;
+        }
+        if (this._switchWorkspaceNotifyId > 0) {
+            global.window_manager.disconnect(this._switchWorkspaceNotifyId);
+            this._switchWorkspaceNotifyId = 0;
+        }
+
+        this.parent();
     }
 });
 
 Signals.addSignalMethods(AppMenuButton.prototype);
 
-// Activities button. Because everything else in the top bar is a
-// PanelMenu.Button, it simplifies some things to make this be one too.
-// We just hack it up to not actually have a menu attached to it.
 const ActivitiesButton = new Lang.Class({
     Name: 'ActivitiesButton',
     Extends: PanelMenu.Button,
 
     _init: function() {
-        this.parent(0.0);
+        this.parent(0.0, null, true);
         this.actor.accessible_role = Atk.Role.TOGGLE_BUTTON;
 
-        let container = new Shell.GenericContainer();
-        container.connect('get-preferred-width', Lang.bind(this, this._containerGetPreferredWidth));
-        container.connect('get-preferred-height', Lang.bind(this, this._containerGetPreferredHeight));
-        container.connect('allocate', Lang.bind(this, this._containerAllocate));
-        this.actor.add_actor(container);
         this.actor.name = 'panelActivities';
 
         /* Translators: If there is no suitable word for "Activities"
            in your language, you can use the word for "Overview". */
         this._label = new St.Label({ text: _("Activities") });
-        container.add_actor(this._label);
+        this.actor.add_actor(this._label);
 
         this.actor.label_actor = this._label;
 
-        this.hotCorner = new Layout.HotCorner();
-        container.add_actor(this.hotCorner.actor);
-
-        // Hack up our menu...
-        this.menu.open = Lang.bind(this, this._onMenuOpenRequest);
-        this.menu.close = Lang.bind(this, this._onMenuCloseRequest);
-        this.menu.toggle = Lang.bind(this, this._onMenuToggleRequest);
-
         this.actor.connect('captured-event', Lang.bind(this, this._onCapturedEvent));
         this.actor.connect_after('button-release-event', Lang.bind(this, this._onButtonRelease));
         this.actor.connect_after('key-release-event', Lang.bind(this, this._onKeyRelease));
 
         Main.overview.connect('showing', Lang.bind(this, function() {
             this.actor.add_style_pseudo_class('overview');
-            this._escapeMenuGrab();
             this.actor.add_accessible_state (Atk.StateType.CHECKED);
         }));
         Main.overview.connect('hiding', Lang.bind(this, function() {
             this.actor.remove_style_pseudo_class('overview');
-            this._escapeMenuGrab();
             this.actor.remove_accessible_state (Atk.StateType.CHECKED);
         }));
 
         this._xdndTimeOut = 0;
     },
 
-    _containerGetPreferredWidth: function(actor, forHeight, alloc) {
-        [alloc.min_size, alloc.natural_size] = this._label.get_preferred_width(forHeight);
-    },
-
-    _containerGetPreferredHeight: function(actor, forWidth, alloc) {
-        [alloc.min_size, alloc.natural_size] = this._label.get_preferred_height(forWidth);
-    },
-
-    _containerAllocate: function(actor, box, flags) {
-        this._label.allocate(box, flags);
-
-        // The hot corner needs to be outside any padding/alignment
-        // that has been imposed on us
-        let primary = Main.layoutManager.primaryMonitor;
-        let hotBox = new Clutter.ActorBox();
-        let ok, x, y;
-        if (actor.get_text_direction() == Clutter.TextDirection.LTR) {
-            [ok, x, y] = actor.transform_stage_point(primary.x, primary.y)
-        } else {
-            [ok, x, y] = actor.transform_stage_point(primary.x + primary.width, primary.y);
-            // hotCorner.actor has northeast gravity, so we don't need
-            // to adjust x for its width
-        }
-
-        hotBox.x1 = Math.round(x);
-        hotBox.x2 = hotBox.x1 + this.hotCorner.actor.width;
-        hotBox.y1 = Math.round(y);
-        hotBox.y2 = hotBox.y1 + this.hotCorner.actor.height;
-        this.hotCorner.actor.allocate(hotBox, flags);
-    },
-
     handleDragOver: function(source, actor, x, y, time) {
         if (source != Main.xdndHandler)
             return DND.DragMotionResult.CONTINUE;
@@ -665,65 +711,36 @@ const ActivitiesButton = new Lang.Class({
         if (this._xdndTimeOut != 0)
             Mainloop.source_remove(this._xdndTimeOut);
         this._xdndTimeOut = Mainloop.timeout_add(BUTTON_DND_ACTIVATION_TIMEOUT,
-                                                 Lang.bind(this, this._xdndShowOverview, actor));
+                                                 Lang.bind(this, this._xdndToggleOverview, actor));
 
         return DND.DragMotionResult.CONTINUE;
     },
 
-    _escapeMenuGrab: function() {
-        if (this.menu.isOpen)
-            this.menu.close();
-    },
-
     _onCapturedEvent: function(actor, event) {
         if (event.type() == Clutter.EventType.BUTTON_PRESS) {
-            if (!this.hotCorner.shouldToggleOverviewOnClick())
+            if (!Main.overview.shouldToggleByCornerOrButton())
                 return true;
         }
         return false;
     },
 
-    _onMenuOpenRequest: function() {
-        this.menu.isOpen = true;
-        this.menu.emit('open-state-changed', true);
-    },
-
-    _onMenuCloseRequest: function() {
-        this.menu.isOpen = false;
-        this.menu.emit('open-state-changed', false);
-    },
-
-    _onMenuToggleRequest: function() {
-        this.menu.isOpen = !this.menu.isOpen;
-        this.menu.emit('open-state-changed', this.menu.isOpen);
-    },
-
     _onButtonRelease: function() {
-        if (this.menu.isOpen) {
-            this.menu.close();
-            Main.overview.toggle();
-        }
+        Main.overview.toggle();
     },
 
     _onKeyRelease: function(actor, event) {
         let symbol = event.get_key_symbol();
         if (symbol == Clutter.KEY_Return || symbol == Clutter.KEY_space) {
-            if (this.menu.isOpen)
-                this.menu.close();
             Main.overview.toggle();
         }
     },
 
-    _xdndShowOverview: function(actor) {
+    _xdndToggleOverview: function(actor) {
         let [x, y, mask] = global.get_pointer();
         let pickedActor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);
 
-        if (pickedActor == this.actor) {
-            if (!Main.overview.visible && !Main.overview.animationInProgress) {
-                Main.overview.showTemporarily();
-                Main.overview.beginItemDrag(actor);
-            }
-        }
+        if (pickedActor == this.actor && Main.overview.shouldToggleByCornerOrButton())
+            Main.overview.toggle();
 
         Mainloop.source_remove(this._xdndTimeOut);
         this._xdndTimeOut = 0;
@@ -733,12 +750,9 @@ const ActivitiesButton = new Lang.Class({
 const PanelCorner = new Lang.Class({
     Name: 'PanelCorner',
 
-    _init: function(box, side) {
+    _init: function(side) {
         this._side = side;
 
-        this._box = box;
-        this._box.connect('style-changed', Lang.bind(this, this._boxStyleChanged));
-
         this.actor = new St.DrawingArea({ style_class: 'panel-corner' });
         this.actor.connect('style-changed', Lang.bind(this, this._styleChanged));
         this.actor.connect('repaint', Lang.bind(this, this._repaint));
@@ -794,12 +808,12 @@ const PanelCorner = new Lang.Class({
         return children[index];
     },
 
-    _boxStyleChanged: function() {
+    setStyleParent: function(box) {
         let side = this._side;
 
-        let rtlAwareContainer = this._box instanceof St.BoxLayout;
+        let rtlAwareContainer = box instanceof St.BoxLayout;
         if (rtlAwareContainer &&
-            this._box.get_text_direction() == Clutter.TextDirection.RTL) {
+            box.get_text_direction() == Clutter.TextDirection.RTL) {
             if (this._side == St.Side.LEFT)
                 side = St.Side.RIGHT;
             else if (this._side == St.Side.RIGHT)
@@ -808,9 +822,9 @@ const PanelCorner = new Lang.Class({
 
         let button;
         if (side == St.Side.LEFT)
-            button = this._findLeftmostButton(this._box);
+            button = this._findLeftmostButton(box);
         else if (side == St.Side.RIGHT)
-            button = this._findRightmostButton(this._box);
+            button = this._findRightmostButton(box);
 
         if (button) {
             if (this._button && this._buttonStyleChangedSignalId) {
@@ -837,7 +851,7 @@ const PanelCorner = new Lang.Class({
 
             // The corner doesn't support theme transitions, so override
             // the .panel-button default
-            button.style = 'transition-duration: 0';
+            button.style = 'transition-duration: 0ms';
         }
     },
 
@@ -850,10 +864,13 @@ const PanelCorner = new Lang.Class({
         let backgroundColor = node.get_color('-panel-corner-background-color');
         let borderColor = node.get_color('-panel-corner-border-color');
 
+        let overlap = borderColor.alpha != 0;
+        let offsetY = overlap ? 0 : borderWidth;
+
         let cr = this.actor.get_context();
         cr.setOperator(Cairo.Operator.SOURCE);
 
-        cr.moveTo(0, 0);
+        cr.moveTo(0, offsetY);
         if (this._side == St.Side.LEFT)
             cr.arc(cornerRadius,
                    borderWidth + cornerRadius,
@@ -862,7 +879,7 @@ const PanelCorner = new Lang.Class({
             cr.arc(0,
                    borderWidth + cornerRadius,
                    cornerRadius, 3 * Math.PI / 2, 2 * Math.PI);
-        cr.lineTo(cornerRadius, 0);
+        cr.lineTo(cornerRadius, offsetY);
         cr.closePath();
 
         let savedPath = cr.copyPath();
@@ -872,14 +889,18 @@ const PanelCorner = new Lang.Class({
         Clutter.cairo_set_source_color(cr, over);
         cr.fill();
 
-        let offset = borderWidth;
-        Clutter.cairo_set_source_color(cr, backgroundColor);
+        if (overlap) {
+            let offset = borderWidth;
+            Clutter.cairo_set_source_color(cr, backgroundColor);
 
-        cr.save();
-        cr.translate(xOffsetDirection * offset, - offset);
-        cr.appendPath(savedPath);
-        cr.fill();
-        cr.restore();
+            cr.save();
+            cr.translate(xOffsetDirection * offset, - offset);
+            cr.appendPath(savedPath);
+            cr.fill();
+            cr.restore();
+        }
+
+        cr.$dispose();
     },
 
     _styleChanged: function() {
@@ -898,6 +919,7 @@ const PANEL_ITEM_IMPLEMENTATIONS = {
     'appMenu': AppMenuButton,
     'dateMenu': imports.ui.dateMenu.DateMenuButton,
     'a11y': imports.ui.status.accessibility.ATIndicator,
+    'a11yGreeter': imports.ui.status.accessibility.ATGreeterIndicator,
     'volume': imports.ui.status.volume.Indicator,
     'battery': imports.ui.status.power.Indicator,
     'lockScreen': imports.ui.status.lockScreenMenu.Indicator,
@@ -927,16 +949,11 @@ const Panel = new Lang.Class({
                                                   reactive: true });
         this.actor._delegate = this;
 
+        this._sessionStyle = null;
+
         this.statusArea = {};
 
-        Main.overview.connect('shown', Lang.bind(this, function () {
-            this.actor.add_style_class_name('in-overview');
-        }));
-        Main.overview.connect('hiding', Lang.bind(this, function () {
-            this.actor.remove_style_class_name('in-overview');
-        }));
-
-        this.menuManager = new PopupMenu.PopupMenuManager(this);
+        this.menuManager = new PopupMenu.PopupMenuManager(this, { keybindingMode: Shell.KeyBindingMode.TOPBAR_POPUP });
 
         this._leftBox = new St.BoxLayout({ name: 'panelLeft' });
         this.actor.add_actor(this._leftBox);
@@ -945,17 +962,10 @@ const Panel = new Lang.Class({
         this._rightBox = new St.BoxLayout({ name: 'panelRight' });
         this.actor.add_actor(this._rightBox);
 
-        if (this.actor.get_text_direction() == Clutter.TextDirection.RTL)
-            this._leftCorner = new PanelCorner(this._rightBox, St.Side.LEFT);
-        else
-            this._leftCorner = new PanelCorner(this._leftBox, St.Side.LEFT);
-
+        this._leftCorner = new PanelCorner(St.Side.LEFT);
         this.actor.add_actor(this._leftCorner.actor);
 
-        if (this.actor.get_text_direction() == Clutter.TextDirection.RTL)
-            this._rightCorner = new PanelCorner(this._leftBox, St.Side.RIGHT);
-        else
-            this._rightCorner = new PanelCorner(this._rightBox, St.Side.RIGHT);
+        this._rightCorner = new PanelCorner(St.Side.RIGHT);
         this.actor.add_actor(this._rightCorner.actor);
 
         this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
@@ -963,8 +973,15 @@ const Panel = new Lang.Class({
         this.actor.connect('allocate', Lang.bind(this, this._allocate));
         this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
 
+        Main.overview.connect('showing', Lang.bind(this, function () {
+            this.actor.add_style_pseudo_class('overview');
+        }));
+        Main.overview.connect('hiding', Lang.bind(this, function () {
+            this.actor.remove_style_pseudo_class('overview');
+        }));
+
         Main.layoutManager.panelBox.add(this.actor);
-        Main.ctrlAltTabManager.addGroup(this.actor, _("Top Bar"), 'start-here-symbolic',
+        Main.ctrlAltTabManager.addGroup(this.actor, _("Top Bar"), 'emblem-system-symbolic',
                                         { sortGroup: CtrlAltTab.SortGroup.TOP });
 
         Main.sessionMode.connect('updated', Lang.bind(this, this._updatePanel));
@@ -1049,6 +1066,9 @@ const Panel = new Lang.Class({
     },
 
     _onButtonPress: function(actor, event) {
+        if (Main.modalCount > 0)
+            return false;
+
         if (event.get_source() != actor)
             return false;
 
@@ -1087,17 +1107,18 @@ const Panel = new Lang.Class({
         return true;
     },
 
-    openAppMenu: function() {
+    toggleAppMenu: function() {
         let indicator = this.statusArea.appMenu;
         if (!indicator) // appMenu not supported by current session mode
             return;
 
         let menu = indicator.menu;
-        if (!indicator.actor.reactive || menu.isOpen)
+        if (!indicator.actor.reactive)
             return;
 
-        menu.open();
-        menu.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
+        menu.toggle();
+        if (menu.isOpen)
+            menu.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
     },
 
     set boxOpacity(value) {
@@ -1121,17 +1142,20 @@ const Panel = new Lang.Class({
         this._updateBox(panel.left, this._leftBox);
         this._updateBox(panel.center, this._centerBox);
         this._updateBox(panel.right, this._rightBox);
-    },
 
-    _initBox: function(elements, box) {
-        for (let i = 0; i < elements.length; i++) {
-            let role = elements[i];
-            let constructor = PANEL_ITEM_IMPLEMENTATIONS[role];
-            if (!constructor) {
-                // panel icon is not supported (can happen for
-                // bluetooth or network)
-                continue;
-            }
+        if (this._sessionStyle)
+            this._removeStyleClassName(this._sessionStyle);
+
+        this._sessionStyle = Main.sessionMode.panelStyle;
+        if (this._sessionStyle)
+            this._addStyleClassName(this._sessionStyle);
+
+        if (this.actor.get_text_direction() == Clutter.TextDirection.RTL) {
+            this._leftCorner.setStyleParent(this._rightBox);
+            this._rightCorner.setStyleParent(this._leftBox);
+        } else {
+            this._leftCorner.setStyleParent(this._leftBox);
+            this._rightCorner.setStyleParent(this._rightBox);
         }
     },
 
@@ -1209,5 +1233,17 @@ const Panel = new Lang.Class({
         this.statusArea[role] = indicator;
         this._addToPanelBox(role, indicator, position, boxContainer);
         return indicator;
+    },
+
+    _addStyleClassName: function(className) {
+        this.actor.add_style_class_name(className);
+        this._rightCorner.actor.add_style_class_name(className);
+        this._leftCorner.actor.add_style_class_name(className);
+    },
+
+    _removeStyleClassName: function(className) {
+        this.actor.remove_style_class_name(className);
+        this._rightCorner.actor.remove_style_class_name(className);
+        this._leftCorner.actor.remove_style_class_name(className);
     }
 });
